import {
  select,
  csv,
  scaleLinear,
  scaleOrdinal,
  extent,
  axisBottom,
  axisLeft,
  schemeCategory10,
} from 'd3';
import { colorLegend } from './colorLegend';

export const scatterPlot = (selection, props) => {
  const {
    margin,
    width,
    height,
    data,
    xValue,
    xAxisLabel,
    yValue,
    yAxisLabel,
    colorValue,
    colorLegendLabel,
  } = props;

  const innerWidth =
    width - margin.right - margin.left-50;
  const innerHeight =
    height - margin.top - margin.bottom;

  const g = selection
    .selectAll('.container')
    .data([null]);
  const gEnter = g
    .enter()
    .append('g')
    .attr('class', 'container');
  gEnter
    .merge(g)
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`
    );

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight])
    .nice();

  const colorScale = scaleOrdinal()
    .domain(data.map(colorValue))
    .range(schemeCategory10);

  const xAxisG = g.select('.x-axis');
  const xAxisGEnter = gEnter
    .append('g')
    .attr('class', 'x-axis');
  xAxisG
    .merge(xAxisGEnter)
    .attr(
      'transform',
      `translate(0,${innerHeight})`
    )
    .call(
      axisBottom(xScale)
        .tickPadding(10)
        .tickSize(-innerHeight)
    );
  const xAxisLabelText = xAxisGEnter
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 60)
    .attr('fill', 'black')
    .merge(xAxisG.select('.axis-label'))
    .attr('x', innerWidth / 2)
    .text(xAxisLabel);

  const yAxisG = g.select('.y-axis');
  const yAxisGEnter = gEnter
    .append('g')
    .attr('class', 'y-axis');
  yAxisG
    .merge(yAxisGEnter)
    .call(
      axisLeft(yScale)
        .tickPadding(10)
        .tickSize(-innerWidth)
    );
  const yAxisLabelText = yAxisGEnter
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -50)
    .attr('transform', 'rotate(-90)')
    .attr('text-anchor', 'middle')
    .attr('fill', 'black')
    .merge(yAxisG.select('.axis-label'))
    .attr('x', -innerHeight / 2)
    .text(yAxisLabel);

selection.call(colorLegend, {
    colorScale,
    colorLegendLabel
  });
  
  const circles = g
    .merge(gEnter)
    .selectAll('circle')
    .data(data);
  circles
    .enter()
    .append('circle')
    .attr('cx', innerWidth / 2)
    .attr('cy', innerHeight / 2)
    .attr('r', 0)
    .merge(circles)
    .transition()
    .duration(2000)
    .delay((d, i) => i * 10)
    .attr('cy', (d) => yScale(yValue(d)))
    .attr('cx', (d) => xScale(xValue(d)))
    .attr('r', 10)
    .attr('fill', (d) =>
      colorScale(colorValue(d))
    );
};
